# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_cons.ipynb.

# %% auto 0
__all__ = ['WRAPS_ASSIGN_ANNDOCS', 'COORD', 'SLICE', 'THWC', 'TCHW', 'TIME_HEIGHT_WIDTH_CHANNELS', 'TIME_CHANNELS_HEIGHT_WIDTH',
           'THWC_AXES', 'TCHW_AXES', 'TIME_HEIGHT_WIDTH_CHANNELS_AXES', 'TIME_CHANNELS_HEIGHT_WIDTH_AXES']

# %% ../nbs/00_cons.ipynb 6
#| export


# %% ../nbs/00_cons.ipynb 8
from typing import TypeVar, ParamSpec, TypeAlias, TypeVarTuple

# %% ../nbs/00_cons.ipynb 10
#| export


# %% ../nbs/00_cons.ipynb 12
from nlit import (
    TCHW, THWC, COORD, SLICE, 
    __DOC__, __MODULE__, __ANNOTATIONS__
)

# %% ../nbs/00_cons.ipynb 14
#| export


# %% ../nbs/00_cons.ipynb 16
WRAPS_ASSIGN_ANNDOCS = (__DOC__, __MODULE__, __ANNOTATIONS__)

# %% ../nbs/00_cons.ipynb 18
COORD = COORD
'''Coordinate format for slice specifications''';

SLICE = SLICE
'''Slice format for slice specifications''';

# %% ../nbs/00_cons.ipynb 20
THWC = THWC
'''The time, height, width, and channels axis order.''';

TCHW = TCHW
'''The time, channels, height, and width axis order.''';

TIME_HEIGHT_WIDTH_CHANNELS = THWC
'''The time, height, width, and channels axis order.''';

TIME_CHANNELS_HEIGHT_WIDTH = TCHW
'''The time, channels, height, and width axis order.''';

THWC_AXES = (0, 1, 2, 3)
'''The time, height, width, and channels axis order as a tuple of integers.''';

TCHW_AXES = (0, 3, 1, 2)
'''The time, channels, height, and width axis order as a tuple of integers.''';

TIME_HEIGHT_WIDTH_CHANNELS_AXES = THWC_AXES
'''The time, height, width, and channels axis order as a tuple of integers.''';

TIME_CHANNELS_HEIGHT_WIDTH_AXES = TCHW_AXES
'''The time, channels, height, and width axis order as a tuple of integers.''';


# %% ../nbs/00_cons.ipynb 22
#| export
