# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/04_time.ipynb.

# %% auto 0
__all__ = ['day2hrs', 'day2hours']

# %% ../nbs/04_time.ipynb 6
from functools import wraps
from datetime import date, datetime

# %% ../nbs/04_time.ipynb 8
#| export


# %% ../nbs/04_time.ipynb 10
#| export


# %% ../nbs/04_time.ipynb 12
from nlit import __DOC__, __NAME__, __MODULE__, __ANNOTATIONS__

# %% ../nbs/04_time.ipynb 14
#| export


# %% ../nbs/04_time.ipynb 16
def day2hrs(day: int) -> int:
    '''Converts a quantity in days to the equivalent number of hours.
    
    Parameters
    ----------
    day : int
        The number of days to convert.
        
    Returns
    -------
    int
        The equivalent number of hours.
        
    See Also
    --------
    day2hours : Alias of `day2hrs`.
    '''
    return int(day * 24)

@wraps(day2hrs, assigned=(__DOC__, __NAME__, __MODULE__, __ANNOTATIONS__))
def day2hours(day: int) -> int:
    '''Alias of `day2hrs`'''
    return day2hrs(day)
